/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include <iostream>
#include <qstring.h>
#include "colorcoding.h"

double colorCoding::f(void)
{ //calculates the resistance value expressed by a given color code
	double R,band1,band2,band3;
	
	R=parameters[0];
	band1=parameters[1];
	band2=parameters[2];
	band3=parameters[3];
    
	if(band3==7) band3=-1; //gold ring
	if(band3==8) band3=-2; //silver ring
	R=(band1*10+band2)*pow(10,band3);
	
	return R;
}

int colorCoding::solve(int index_to_solve)
{
	double R,band1,band2,band3;
	double exponent,first_two_digits;
	
	if(index_to_solve!=0) //we know the resistance, we look for the color coding
	{
		//there was a very nasty bug with calculations like 1/0.1: on PPC->10 on intel: 9.9999999
		//so the floor() was returning 10 on ppc (which is correct) and 9 on intel...
		//this is because floating point arithmetic can not be precise for numbers between 1 and 0
		//there is no exact match between a decimal number and its binary representation in that range
		//so, adding 1/1000th to R is not going to change anything 
		// (it will be lost at the moment we have exponent and first_two_digits)
		//but it allows for some loss of precision in the division without being too far from the correct result
		// (the result will be slightly too big, but the floor will cut that off)
		R=1.001*parameters[0];
		exponent=floor(log10(R))-1;
		first_two_digits=floor(R/pow(10,exponent));
		
		//R will be represented as DDx10^E. We try to round DD to the best value in order to be as close to R as possible
		if(fabs(R-(first_two_digits*pow(10,exponent))) > fabs(R-((first_two_digits+1)*pow(10,exponent))) ) first_two_digits += 1;
		
		if(first_two_digits==100) {
			exponent += 1;
			first_two_digits=first_two_digits/10.0;
		}
		
		if(first_two_digits/10.0==floor(first_two_digits/10.0)) 
		//we have a low precision requirement: trying to represent R as Dx10^E
		{
			band1=0;
			band2=first_two_digits/10;
			band3=exponent+1;
			if(band3==7){ //slight overflow: result can not be expressed as Dx10^E
				band1=band2;
				band2=0;
				band3=6;
			}
		} else //representing R as DDx10^E
		{
			band1=floor(first_two_digits/10);
			band2=first_two_digits-band1*10;
			band3=exponent;
			if(band3==-3){ //slight underflow: result can not be expressed as DDx10^E but needs to be Dx10^E
				if(band2<5) band2=band1; else band2=band1+1; //we try to round off cleverly: 28m->30m, 24m->20m
				if(band2>9) {
					band2=0;
					band1=1;
				} else band1=0;
				band3=-2;
			}
		}
		
		//printf("%.0g - %.0g - %.0g -> for %g asked\n\n",band1,band2,band3,parameters[0]);
		if(band3>6) { //overflow: no higher multiplier than 1e6
			band3=6;
			band2=9;
			band1=9;
		}
		if(band3<-2) { //underflow: no smaller multiplier than 1e-2
			band3=8;
			band2=1;
			band1=0;
		}
		if(band3<-1) band3=8; //silver ring
		if(band3<0 && band3>=-1) band3=7; //gold ring
		parameters[1]=band1;
		parameters[2]=band2;
		parameters[3]=band3;
	} //if index_to_solve==0, we go directly here
	
	R=f();
	parameters[0]=R;
	
	return EXIT_SUCCESS;
}
