/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifndef EQUATION_H
#define EQUATION_H
#include <qstring.h>
#include <math.h>
#include "pptools.h"

#define PI 3.141592653589
#define MU_0 1.25663706e-6
#define E_0 8.8541878176e-12

#define MAX_TAB_N_PTS 2000

/* This class is for managing extracting data points from tabulated values, doing interpolations, ... */
class tabulation
{
public:
	double value;
	enum InterpType {InterpLinear, InterpXLog, InterpYLog, InterpXYLog};
public:
	int getValue(double x,unsigned short int interp_type);
	int setTables(double Xt[],double Yt[],short int nb_elem);
private:
	double X[MAX_TAB_N_PTS],Y[MAX_TAB_N_PTS];
	short int N_ELEM;
};

/* This class is for working with equations (Newton's method, ...) */
const double DeltaX=1e-3;		//derivative calculated from F(x(1-DeltaX/2)) and F(x(1+DeltaX/2))
const double TolConv=1e-4;		//tolerance for the Newton's method
#define MAX_EQ_ITERATIONS 1e4
#define MAX_EQ_PARAMETERS 127

class equation 
{
public:
	QString message;
	enum FieldFlags {GRT_INDEX_MASK=127, IS_GRT_THAN=128, IS_SM_THAN=256, IS_HIDDEN=512, IS_INTEGER=1024, IS_POSITIVE=2048, IS_NEGATIVE=4096};
public:
	virtual ~equation() { }
	virtual double f(void); //in fact, F must be f(x)-y=0, so solving it for 0 finds x such as y=f(x)
	double df(int index_deri);
	int Newton(int index_to_solve,double guess);
	virtual int solve(int index_to_solve);
public:
	double parameters[MAX_EQ_PARAMETERS];
	double guess[MAX_EQ_PARAMETERS]; //initial values of the parameters for the solvers
	int constraints[MAX_EQ_PARAMETERS]; //solving constraints
protected:
	short int nb_params;
};

#endif
