/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "forms.h"
 
int forms::addWidgets(QLineEdit *input,QLabel *name,QLabel *unit) {
//add a line of widgets to a form
	nb_elems++;
	form_input[nb_elems]=input;
	form_labels[nb_elems][0]=name;
	form_labels[nb_elems][1]=unit;
	
	return EXIT_SUCCESS;
}

int forms::addLabels(short int selected_set,short int position,QString name,QString tooltip,QString unit,int flags,double guess) {
//add labels to a form, given its set number
	labels_text[selected_set][position][0]=name;
	labels_text[selected_set][position][1]=tooltip;
	labels_text[selected_set][position][2]=unit;
	fields_flags[selected_set][position]=flags;
	form_guess[selected_set][position]=guess;
	return EXIT_SUCCESS;
}

int forms::refreshForm(short int selected_set) {
//redisplay a form, depending on the set that is selected
	int i;
	
	current_set=selected_set;
	for(i=1;i<=nb_elems;i++) { //when flag field ok -> if HIDDEN, go to the next one
		if((fields_flags[current_set][i]&equation::IS_HIDDEN) == 0) {
			form_labels[i][0]->show();
			form_input[i]->show();
			form_labels[i][1]->show();
			form_labels[i][0]->setText(labels_text[selected_set][i][0]+":");
			QToolTip::add(form_input[i],labels_text[selected_set][i][1]);
			form_labels[i][1]->setText(labels_text[selected_set][i][2]);
		} else {
			form_labels[i][0]->hide();
			form_input[i]->hide();
			form_labels[i][1]->hide();
		}
	}
	
	return EXIT_SUCCESS;
}

int forms::EngToDec(QString *value) {
//convert from engineering notation to fully decimal
	const short int index=11;
	QChar suffixes[index]={'f', 'p', 'n', 'u', 'm', 'k', 'K', 'M', 'G', 'T', 'P'};
	QString values[index]= {"e-15", "e-12", "e-9", "e-6", "e-3", "e3", "e3", "e6", "e9", "e12", "e15"};
	short int i;
	
	for(i=0;i<index;i++) value->replace(suffixes[i],values[i]);
	
	return EXIT_SUCCESS;
}

QString forms::DecToEng(double value) {
//convert from decimal notation to engineering
	const short int index=13;
	QChar suffixes[index]={'f', 'p', 'n', 'u', 'm', ' ', 'k', 'K', 'M', 'G', 'T', 'P', ' '};
	double values[index]= {1e-15, 1e-12, 1e-9, 1e-6, 1e-3, 1e0, 1e3, 1e3, 1e6, 1e9, 1e12, 1e15, 1e18};
	short int i,range;
	QString result;
	
	range=-1;
	i=0;
	do {
		if(value>=values[i] && value<values[i+1]) range=i;
		i++;
		
	} while(i<(index) && range==-1);
	
	if(value<values[0] || value>values[index-1]) {
		result.setNum(value,'g',PRECISION);
	} else {
		value=value/values[range];
		result.setNum(value,'g',PRECISION);
		if(suffixes[range]!=' ') result.append(suffixes[range]);
	}
	
	return result;
}

int forms::getValues(double vals[],int index_to_solve[],int vals_constraints[],double guess[],short int nb_index) {
//read the values from the input fields of the form and put them into a table
//it also gives a tables containing the indexes of parameters that are unknown
	int i_elem,solve_index;
	QString buffer;
	bool ok;
	
	for(solve_index=0;solve_index<nb_index;solve_index++) {
	//initialize index_to_solve to last fields -> we solve the main parameter(s) if everything is already given
		index_to_solve[solve_index]=nb_elems-solve_index;
	}
	solve_index=1;
	for(i_elem=1;i_elem<=nb_elems;i_elem++) { 
	//when flag field ok -> if HIDDEN, go to the next one
		if((fields_flags[current_set][i_elem]&equation::IS_HIDDEN) == 0) {
			buffer=form_input[i_elem]->text();
			if(buffer.isEmpty()) {
				if(solve_index > nb_index) {
					message.append("Too many unknown values\n");
					return EXIT_FAILURE;
				}
				index_to_solve[solve_index]=i_elem;
				solve_index++;
			}
			else {
				EngToDec(&buffer);
				vals[i_elem]=buffer.toDouble(&ok);
				if(ok==FALSE) {
					message=QString("Wrong format for \"%1\"\n").arg(labels_text[current_set][i_elem][0]);
					return EXIT_FAILURE;
				}
			}
		} else {
			vals[i_elem]=0; //this element is hidden anyway...
		}
		//we fill out all these fields, even if hidden (easiest processing later)
		vals_constraints[i_elem]=fields_flags[current_set][i_elem];
		guess[i_elem]=form_guess[current_set][i_elem];
	}
	index_to_solve[0]=solve_index-1; //so it contains the number of parameters that have to be solved
	
	return EXIT_SUCCESS;
}

int forms::setValues(double table[]) {
//puts the values given in table into the input fields of the form
	int i_elem;
	
	for(i_elem=1;i_elem<=nb_elems;i_elem++) { 
		form_input[i_elem]->setText(DecToEng(table[i_elem]));
	}
	return EXIT_SUCCESS;
}

