/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifndef INDUCTANCE_H
#define INDUCTANCE_H
#include "equation.h"

/* this class is used for inductances calculations */
class inductance : public equation 
{
public:
	double L;
	double C;
	QString message;
public:
	inductance();
	int setType(int choice);
	double f(void);
	void CalculateLandC(void);
	double hollow_cylinder(void);
	double wire_loop(void);
	double thin_solenoid(void);
	double torus(void);
	double parallele_wires(void);
	double coaxial_line(void);
	double strip_line(void);
	double converging_line(void);
	double disks(void);
	double rectangular_coil(void);
	double helix(void);
	double balls(void);
	double L2C(void);
	double C2L(void);
private:
	int setTabKsh(void);
	int setTabKl(void);
	int setTabN(void);
	int setTabG(void);
	int setTabKw(void);
	double Ksh(double a);
	double Kl(double a);
	double Ns(double a);
	double Gs(double a);
	double Kw(double a);
private:
	enum InductanceType {ParalleleWires, StripLine, ConvergingLine, Disks, HollowCylinder, CoaxialLine, ThinSolenoid, Torus, WireLoop, RectangularCoil, Helix, Balls};
	InductanceType type;
	tabulation tab_Ksh;
	tabulation tab_Kl;
	tabulation tab_N;
	tabulation tab_G;
	tabulation tab_Kw;
	double L_length;
};
#endif
