/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "liquidresistor.h"

liquidResistor::liquidResistor()
{
	nb_params=3;
	message="";
}

void liquidResistor::setSolution(QString choice)
{
	//data from CRC, p5-94 for Lambda0 and resistivities,
	//CRC p8-108 for maximum solubility at 20C
	const double rho_h2o=998.21;	//at 20C, in g/l
	double mass_dissolution=99.999999;	//at 20C,mass percent of solute (max. solubility)
	const short int Npts=7;
	double Xcuso4[Npts] = {
	0.0005, 0.001, 0.005, 0.01, 0.02, 0.05, 0.1
	};
	double Ycuso4[Npts] = {
	121.6, 115.20, 94.02, 83.08, 72.16, 59.02, 50.55
	};
	double Ynacl[Npts] = {
	124.44, 123.68, 120.59, 118.45, 115.70, 111.01, 106.69
	};
	double Yznso4[Npts] = {
	121.3, 114.47, 95.44, 84.87, 74.20, 61.17, 52.61
	};
        double Ynh4cl[Npts] = {
        147.5, 146.7, 134.4, 141.21, 138.25, 133.22, 128.69
        };
	const short int NptsKmno4=4;
	double Xkmno4[NptsKmno4] = {
	0.0005, 0.001, 0.01, 0.1
	};
	double Ykmno4[NptsKmno4] = {
	132.7, 131.9, 126.5, 113
	};
	
	if(choice=="KMnO4") { //potassium permanganate
		Amol=39.0983+54.93805+4*15.9994;
		Lambda0=134.8;
		tab_lambda.setTables(Xkmno4,Ykmno4,NptsKmno4);
		mass_dissolution=5.96;
	}
	if(choice=="CuSO4") { //copper sulfate
		Amol=63.546+32.066+4*15.9994;
		Lambda0=133.6;
		tab_lambda.setTables(Xcuso4,Ycuso4,Npts);
		mass_dissolution=16.7;
	}
	if(choice=="ZnSO4") { //zinc sulfate
		Amol=65.39+32.066+4*15.9994;
		Lambda0=132.7;
		tab_lambda.setTables(Xcuso4,Yznso4,Npts);
		mass_dissolution=35.0;
	}
	if(choice=="NaCl") { //salt
		Amol=22.989768+35.4527;
		Lambda0=126.39;
		tab_lambda.setTables(Xcuso4,Ynacl,Npts);
		mass_dissolution=26.41;
	}
        if(choice=="NH4Cl") { //ammonium chloride
		Amol=14.00674+4*1.00794+35.4527;
		Lambda0=149.6;
		tab_lambda.setTables(Xcuso4,Ynh4cl,Npts);
		mass_dissolution=28.34;
	}
	MaxDissolution=(rho_h2o)/(100.0/mass_dissolution-1);

}

double liquidResistor::f(void)
{
	int status;
	double R,concentration,length,diameter,rho, lambda;
	
	diameter=parameters[1];
	length=parameters[2];
	concentration=parameters[3];
	R=parameters[4];
	
	if((concentration/Amol)>=0.0005) {
		status=tab_lambda.getValue(concentration/Amol,tabulation::InterpXLog);
		if(status==EXIT_FAILURE) message="Error, calculation is wrong !";
		if(status==2) message.append("\nOut of table interpolation");
		lambda=tab_lambda.value*1.0e-4; //weird units in the CRC
		
	} else {
		lambda=(Lambda0-(60.20+0.229*Lambda0)*pow(concentration/Amol,0.5))*1.0e-4; //cf CRC p5-94
		message.append("\nOut of table interpolation, using simplified model");
	}
	rho=1.0/(lambda*concentration/Amol*1000); //lambda in m^2*S/mol, Concentration in g/L and Amol in g/mol
	
	return R-rho*length/(PI*diameter*diameter/4);
}

