/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifndef RLC_H
#define RLC_H
#include <qstring.h>
#include <iostream>
#include "equation.h"
#include "pptools.h"

const int MAXSTEPS=10000;	//if we don't have the result after that many steps, we abandon
const double TOLERANCE=1e-5;	//requested tolerance for the solver
const double STORED2OHM=0.5;	//fraction of the stored energy that is dissipated into joule heating (for initial guess only)
const double ALPHA=0.01; 	//range for calculating the derivatives
const double MINIMUM_CORRECTION=1e-30;	//if the corrective factor is less that that, we consider that it is converging too slowly

class rlc : public equation 
{
public:
	rlc() {nb_params=6;}
	int solve(int index_to_solve[3]);
	double F(void);
	double U(void);
	double f(void);
private:
	double F_0(void);
	double U_0(void);
	double dF_0(int index_variable);
	double dU_0(int index_variable);
	int solve_system(int index_to_solve1,int index_to_solve2,double guess1,double guess2);
	int setPhysicalGuesses(int index_to_solve[3],double guesses[3]);
	int setRLCParameters(int index_to_solve[3]);
private:
	double k;
	double R,L,C,Imax,tmax,U0;
	enum EqTypes {fIsF0, fIsF, fIsU0, fIsU};
	enum EqTypes eqselection;
};
#endif
