/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/****************************************************************************
** ui.h extension file, included from the uic-generated form implementation.
**
** If you want to add, delete, or rename functions or slots, use
** Qt Designer to update this file, preserving your code.
**
** You should not define a constructor or destructor in this file.
** Instead, write your code in functions called init() and destroy().
** These will automatically be called by the form's constructor and
** destructor.
*****************************************************************************/

#include <qstring.h>
#include <qlineedit.h>
#include <qtooltip.h>
#include "pptools.h"
#include "forms.h"
#include "equation.h"
#include "rlc.h"
#include "inductance.h"
#include "liquidresistor.h"
#include "skindepth.h"
#include "colorcoding.h"
#include "help.h"

int RLCwindow::solve_rlc()
{
	rlc circuit;
	int index_to_solve[3];
	
	if(rlc_input_form.getValues(circuit.parameters,index_to_solve,circuit.constraints,circuit.guess,2)==EXIT_FAILURE) {
		text_status->setText(rlc_input_form.message);
		return EXIT_FAILURE;
	}
	
	circuit.solve(index_to_solve);
	text_status->setText(circuit.message);
	rlc_input_form.setValues(circuit.parameters);
	
	return EXIT_SUCCESS;
}


int RLCwindow::solve_liquid_R()
{
	int index_to_solve[2];
	liquidResistor lR;
	
	index_to_solve[1]=0;
    	if(lR_input_form.getValues(lR.parameters,index_to_solve,lR.constraints,lR.guess,1)==EXIT_FAILURE) {
		printf("%s\n",lR_input_form.message.ascii());
		return EXIT_FAILURE;
	}

	lR.setSolution(solution_select->currentText());
	lR.solve(index_to_solve[1]);
	if(lR.parameters[3]>lR.MaxDissolution) {
		//we can not dissolve that much product->show what is the maximum
		lR.parameters[3]=lR.MaxDissolution;
		lR.message.append("Maximum dissolution reached!\n");
		lR.solve(4);
	}
	printf("%s\n",lR.message.ascii());
	
	lR_input_form.setValues(lR.parameters);
	
	return EXIT_SUCCESS;
}


int RLCwindow::solve_color_coding()
{
	int index_to_solve[2];
	colorCoding code;
	
	index_to_solve[1]=0;
	//that test is not needed anymore: if we reached that function, that means we are looking for the color coding for a given R
	//but we need that call in order to use the IngToDec call, ...
	//basically, this IngToDec call is here more a hack that anything clean...
	if(color_code_input_form.getValues(code.parameters,index_to_solve,code.constraints,code.guess,1)==EXIT_FAILURE) {
		printf("%s\n",color_code_input_form.message.ascii());
		return EXIT_FAILURE;
	}
	
	//in the form, R is in first position while in the parameters it needs to be in position 0
	index_to_solve[1]--;
	if(index_to_solve[1]==0) code.parameters[0]=0;
	else code.parameters[0]=code.parameters[1];

	code.parameters[1]=(float)band1_input->currentItem();
	code.parameters[2]=(float)band2_input->currentItem();
	code.parameters[3]=(float)band3_input->currentItem();

	code.solve(index_to_solve[1]);
	
	band1_input->setCurrentItem((int)code.parameters[1]);
	band2_input->setCurrentItem((int)code.parameters[2]);
	band3_input->setCurrentItem((int)code.parameters[3]);

	code.parameters[1]=code.parameters[0];
	color_code_input_form.setValues(code.parameters);
	
	return EXIT_SUCCESS;
}

int RLCwindow::calculate_color_coding()
{
	int index_to_solve[2];
	colorCoding code;
	
	index_to_solve[1]=0;
	code.parameters[0]=0;
	
	code.parameters[1]=(float)band1_input->currentItem();
	code.parameters[2]=(float)band2_input->currentItem();
	code.parameters[3]=(float)band3_input->currentItem();

	code.solve(index_to_solve[1]);
	
	band1_input->setCurrentItem((int)code.parameters[1]);
	band2_input->setCurrentItem((int)code.parameters[2]);
	band3_input->setCurrentItem((int)code.parameters[3]);

	code.parameters[1]=code.parameters[0];
	color_code_input_form.setValues(code.parameters);
	
	return EXIT_SUCCESS;

}


int RLCwindow::solve_inductance()
{
	inductance indu;
	int index_to_solve[3];

	//setting the selecting inductance geometry
	indu.setType(inductance_type_select->currentItem());
	if(L_input_form.getValues(indu.parameters,index_to_solve,indu.constraints,indu.guess,2)==EXIT_FAILURE) {
		printf("%s\n",L_input_form.message.ascii());
		return EXIT_FAILURE;
	}
	if(index_to_solve[0]==2) {
		//anything but L and C
		if(index_to_solve[1]!=5 && index_to_solve[2]==6) {
			index_to_solve[0]=1;
			indu.L2C();
		}
		//anything and L
		if(index_to_solve[2]==5) {
			index_to_solve[0]=1;
			indu.C2L();
		}
		//L and C
		if(index_to_solve[1]==5 && index_to_solve[2]==6) {
			index_to_solve[0]=1;
		}
		if(index_to_solve[0]>1) {
			printf("Too many unknown!\n");
			return EXIT_FAILURE;
		}
	}

	if(index_to_solve[1]==5) indu.CalculateLandC();
	else {
		indu.solve(index_to_solve[1]);
		indu.CalculateLandC(); //enables checking the consistency of the results
	}
	
	//displaying the results
	L_input_form.setValues(indu.parameters);
	L_status_text->setText(indu.message);
	
	return EXIT_SUCCESS;
}


int RLCwindow::refresh_inductance()
{
	L_input_form.refreshForm(inductance_type_select->currentItem()+1);
	return EXIT_SUCCESS;	
}


int RLCwindow::solve_skin_depth()
{
	int index_to_solve[2];
	skinDepth sk;
	
    	if(sk_input_form.getValues(sk.parameters,index_to_solve,sk.constraints,sk.guess,1)==EXIT_FAILURE) {
		printf("%s\n",sk_input_form.message.ascii());
		return EXIT_FAILURE;
	}
	
	sk.setParameters(select_waveform->currentText(),select_material->currentText());
	sk.solve(index_to_solve[1]);
	
	text_skin_depth->setText(sk.message);
	sk_input_form.setValues(sk.parameters);
	
	return EXIT_SUCCESS;
}

void RLCwindow::HelpOpen() {
    Help dlg( this, 0, TRUE );
    dlg.exec();
}

void RLCwindow::init() {
	
	//initializing RLC form
	rlc_input_form.addWidgets(rlc_input_1,rlc_name_1,rlc_unit_1);
	rlc_input_form.addWidgets(rlc_input_2,rlc_name_2,rlc_unit_2);
	rlc_input_form.addWidgets(rlc_input_3,rlc_name_3,rlc_unit_3);
	rlc_input_form.addWidgets(rlc_input_4,rlc_name_4,rlc_unit_4);
	rlc_input_form.addWidgets(rlc_input_5,rlc_name_5,rlc_unit_5);
	rlc_input_form.addWidgets(rlc_input_6,rlc_name_6,rlc_unit_6);
	rlc_input_form.addLabels(1,1,"tmax","rise time","s",equation::IS_POSITIVE,1e-6);
	rlc_input_form.addLabels(1,2,"Imax","maximum current reached at tmax","A",equation::IS_POSITIVE,1e3);
	rlc_input_form.addLabels(1,3,"U0","charging voltage","V",equation::IS_POSITIVE,1e3);
	rlc_input_form.addLabels(1,4,"R","circuit resistance","&Omega;",equation::IS_POSITIVE,1e-3);
	rlc_input_form.addLabels(1,5,"L","circuit inductance","H",equation::IS_POSITIVE,1e-9);
	rlc_input_form.addLabels(1,6,"C","circuit capacitance","F",equation::IS_POSITIVE,1e-6);
	//rlc_input_form.refreshForm(1);
	
	//initializing color coding form
	color_code_input_form.addWidgets(ccoding_input,ccoding_name,ccoding_unit);
	color_code_input_form.addLabels(1,1,"Resistance","Value of the resistor","&Omega;",equation::IS_POSITIVE,1e-6);
	
	//initializing liquid resistor form
	lR_input_form.addWidgets(lR_input_1,lR_name_1,lR_unit_1);
	lR_input_form.addWidgets(lR_input_2,lR_name_2,lR_unit_2);
	lR_input_form.addWidgets(lR_input_3,lR_name_3,lR_unit_3);
	lR_input_form.addWidgets(lR_input_4,lR_name_4,lR_unit_4);
	lR_input_form.addLabels(1,1,"Diameter","inside diameter of the resistance","m",equation::IS_POSITIVE,1e-3);
	lR_input_form.addLabels(1,2,"Length","inside length of the resistance","m",equation::IS_POSITIVE,1e-2);
	lR_input_form.addLabels(1,3,"Concentration","concentration of the solution making the resistance","kg/l",equation::IS_POSITIVE,1);
	lR_input_form.addLabels(1,4,"Resistance","resistance","ohm",equation::IS_POSITIVE,1e3);
	//lR_input_form.refreshForm(1);
	
	//initializing skin depth form
	sk_input_form.addWidgets(skin_input_1,skin_name_1,skin_unit_1);
	sk_input_form.addWidgets(skin_input_2,skin_name_2,skin_unit_2);
	sk_input_form.addWidgets(skin_input_3,skin_name_3,skin_unit_3);
	sk_input_form.addLabels(1,1,"Rise time","rise time of the signal","s",equation::IS_POSITIVE,1e-6);
	sk_input_form.addLabels(1,2,"Current time","time at which we want to calculate/we know the skin depth","s",equation::IS_POSITIVE,1e-7);
	sk_input_form.addLabels(1,3,"skin depth","distance such as B(d)=B0/e","m",equation::IS_POSITIVE,1e-6);
	//sk_input_form.refreshForm(1);
	
	//initializing inductance form
	L_input_form.addWidgets(L_input_1,L_label_1,L_unit_1);
	L_input_form.addWidgets(L_input_2,L_label_2,L_unit_2);
	L_input_form.addWidgets(L_input_3,L_label_3,L_unit_3);
	L_input_form.addWidgets(L_input_4,L_label_4,L_unit_4);
	L_input_form.addWidgets(L_input_5,L_label_5,L_unit_5);
	L_input_form.addWidgets(L_input_6,L_label_6,L_unit_6);
	
	//parallele wires
	L_input_form.addLabels(1,1,"wire diameter","diameter of the wire","m",equation::IS_POSITIVE|equation::IS_SM_THAN|2,0.99);
	L_input_form.addLabels(1,2,"gap","gap between the two wires","m",equation::IS_POSITIVE|equation::IS_GRT_THAN|1,1.01);
	L_input_form.addLabels(1,3,"length","length of the wires","m",equation::IS_POSITIVE,1e-2);
	L_input_form.addLabels(1,4,"","","",equation::IS_HIDDEN,1);
	L_input_form.addLabels(1,5,"L","inductance","H",equation::IS_POSITIVE,1e-9);
	L_input_form.addLabels(1,6,"C","capacitance","F",equation::IS_POSITIVE,1e-9);
	
	//strip line
	L_input_form.addLabels(2,1,"width","width of the strip line","m",equation::IS_POSITIVE,1e-2);
	L_input_form.addLabels(2,2,"gap","gap between the two electrodes","m",equation::IS_POSITIVE,1e-3);
	L_input_form.addLabels(2,3,"length","length of the strip line","m",equation::IS_POSITIVE,1e-2);
	L_input_form.addLabels(2,4,"","","",equation::IS_HIDDEN,1);
	L_input_form.addLabels(2,5,"L","inductance","H",equation::IS_POSITIVE,1e-9);
	L_input_form.addLabels(2,6,"C","capacitance","F",equation::IS_POSITIVE,1e-9);
	
	//converging line
	L_input_form.addLabels(3,1,"small width","width of the smallest side of the strip line","m",equation::IS_POSITIVE|equation::IS_SM_THAN|4,0.99);
	L_input_form.addLabels(3,2,"gap","gap between the two electrodes","m",equation::IS_POSITIVE,1e-3);
	L_input_form.addLabels(3,3,"length","length of the strip line","m",equation::IS_POSITIVE,1e-2);
	L_input_form.addLabels(3,4,"large width","width of the largest side of the strip line","m",equation::IS_POSITIVE|equation::IS_GRT_THAN|1,1.01);
	L_input_form.addLabels(3,5,"L","inductance","H",equation::IS_POSITIVE,1e-9);
	L_input_form.addLabels(3,6,"C","capacitance","F",equation::IS_POSITIVE,1e-9);
	
	//circular plates
	L_input_form.addLabels(4,1,"inner radius 1","smallest radius of the circular electrodes","m",equation::IS_POSITIVE|equation::IS_SM_THAN|2,0.99);
	L_input_form.addLabels(4,2,"outer radius 2","largest radius of the circular electrodes","m",equation::IS_POSITIVE|equation::IS_GRT_THAN|1,1.01);
	L_input_form.addLabels(4,3,"gap 1","gap at the inner radius","m",equation::IS_POSITIVE,1e-2);
	L_input_form.addLabels(4,4,"gap 2","gap at the outer radius","m",equation::IS_POSITIVE,1e-2);
	L_input_form.addLabels(4,5,"L","inductance","H",equation::IS_POSITIVE,1e-9);
	L_input_form.addLabels(4,6,"C","capacitance","F",equation::IS_POSITIVE,1e-9);
	
	//hollow cylinder
	L_input_form.addLabels(5,1,"inside radius","inside radius of the cylinder","m",equation::IS_POSITIVE|equation::IS_SM_THAN|2,0.99);
	L_input_form.addLabels(5,2,"outside radius","external radius of the cylinder","m",equation::IS_POSITIVE|equation::IS_GRT_THAN|1,1.01);
	L_input_form.addLabels(5,3,"length","length of the conductor","m",equation::IS_POSITIVE,1e-2);
	L_input_form.addLabels(5,4,"","","",equation::IS_HIDDEN,1);
	L_input_form.addLabels(5,5,"L","inductance","H",equation::IS_POSITIVE,1e-9);
	L_input_form.addLabels(5,6,"C","capacitance","F",equation::IS_POSITIVE,1e-9);
	
	//coaxial conductor
	L_input_form.addLabels(6,1,"inside radius","inside conductor radius","m",equation::IS_POSITIVE|equation::IS_SM_THAN|2,0.99);
	L_input_form.addLabels(6,2,"outside radius","outside conductor radius","m",equation::IS_POSITIVE|equation::IS_GRT_THAN|1,1.01);
	L_input_form.addLabels(6,3,"length","length of the coaxial line","m",equation::IS_POSITIVE,1e-2);
	L_input_form.addLabels(6,4,"","","",equation::IS_HIDDEN,1);
	L_input_form.addLabels(6,5,"L","inductance","H",equation::IS_POSITIVE,1e-9);
	L_input_form.addLabels(6,6,"C","capacitance","F",equation::IS_POSITIVE,1e-9);
	
	//thin solenoid
	L_input_form.addLabels(7,1,"solenoid diameter","diameter of the solenoid","m",equation::IS_POSITIVE,1e-2);
	L_input_form.addLabels(7,2,"number of turns","number of turns of the solenoid","m",equation::IS_POSITIVE,1);
	L_input_form.addLabels(7,3,"length","length of the solenoid","",equation::IS_POSITIVE,1e-2);
	L_input_form.addLabels(7,4,"","","",equation::IS_HIDDEN,1);
	L_input_form.addLabels(7,5,"L","inductance","H",equation::IS_POSITIVE,1e-9);
	L_input_form.addLabels(7,6,"C","capacitance","F",equation::IS_POSITIVE,1e-9);
	
	//torus
	L_input_form.addLabels(8,1,"small diameter","diameter of the bended cylinder making the torus","m",equation::IS_POSITIVE|equation::IS_SM_THAN|3,0.99);
	L_input_form.addLabels(8,2,"number of turns","number of turns of the wire around the torus","m",equation::IS_POSITIVE,1);
	L_input_form.addLabels(8,3,"big diameter","diameter of the torus","m",equation::IS_POSITIVE|equation::IS_GRT_THAN|1,1.01);
	L_input_form.addLabels(8,4,"","","",equation::IS_HIDDEN,1);
	L_input_form.addLabels(8,5,"L","inductance","H",equation::IS_POSITIVE,1e-9);
	L_input_form.addLabels(8,6,"C","capacitance","F",equation::IS_POSITIVE,1e-9);
	
	//wire loop
	L_input_form.addLabels(9,1,"wire diameter","diameter of the wire making the loop","m",equation::IS_POSITIVE|equation::IS_SM_THAN|2,0.99);
	L_input_form.addLabels(9,2,"loop diameter","diameter of the loop","m",equation::IS_POSITIVE|equation::IS_GRT_THAN|1,1.01);
	L_input_form.addLabels(9,3,"","","",equation::IS_HIDDEN,1);
	L_input_form.addLabels(9,4,"","","",equation::IS_HIDDEN,1);
	L_input_form.addLabels(9,5,"L","inductance","H",equation::IS_POSITIVE,1e-9);
	L_input_form.addLabels(9,6,"C","capacitance","F",equation::IS_POSITIVE,1e-9);
	
	//rectangular coil
	L_input_form.addLabels(10,1,"width","width of the coil","m",equation::IS_POSITIVE,1e-2);
	L_input_form.addLabels(10,2,"height","height of the coil","m",equation::IS_POSITIVE,1e-2);
	L_input_form.addLabels(10,3,"length","length of the coil","m",equation::IS_POSITIVE,1e-2);
	L_input_form.addLabels(10,4,"","","",equation::IS_HIDDEN,1);
	L_input_form.addLabels(10,5,"L","inductance","H",equation::IS_POSITIVE,1e-9);
	L_input_form.addLabels(10,6,"C","capacitance","F",equation::IS_POSITIVE,1e-9);
	
	//helix
	L_input_form.addLabels(11,1,"radius","radius of the helix","m",equation::IS_POSITIVE,1e-2);
	L_input_form.addLabels(11,2,"height","height of the helix","m",equation::IS_POSITIVE,1e-2);
	L_input_form.addLabels(11,3,"wire radius","radius of the wire making the coil","m",equation::IS_POSITIVE,1e-3);
	L_input_form.addLabels(11,4,"pitch","pitch of the helix (length between two turns)","m",equation::IS_POSITIVE,1e-2);
	L_input_form.addLabels(11,5,"L","inductance","H",equation::IS_POSITIVE,1e-9);
	L_input_form.addLabels(11,6,"C","capacitance","F",equation::IS_POSITIVE,1e-9);
	
	//balls
	L_input_form.addLabels(12,1,"diameter 1","diameter of ball 1","m",equation::IS_POSITIVE|equation::IS_SM_THAN|3,0.99);
	L_input_form.addLabels(12,2,"diameter 2","diameter of ball 2","m",equation::IS_POSITIVE|equation::IS_SM_THAN|3,0.99);
	L_input_form.addLabels(12,3,"distance","distance between the two balls","m",equation::IS_POSITIVE|equation::IS_GRT_THAN|1,1.01);
	L_input_form.addLabels(12,4,"",")","",equation::IS_HIDDEN,1);
	L_input_form.addLabels(12,5,"L","inductance","H",equation::IS_POSITIVE,1e-9);
	L_input_form.addLabels(12,6,"C","capacitance","F",equation::IS_POSITIVE,1e-9);
	L_input_form.refreshForm(1);
}

