/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "skindepth.h"

int skinDepth::setTabRamp(void) 
{
	const short int NUsual = 21;
	double XUsual[NUsual] = {
	0.0024, 0.0048, 0.0072, 0.012, 0.0192, 0.0288, 0.0432, 0.0624, 0.0888, 0.122, 0.166, 0.221, 0.293, 0.394, 0.737, 0.852, 0.948, 1.03, 1.1, 1.16, 1.2
	};
	double YUsual[NUsual] = {
	0.0444, 0.0629, 0.077, 0.0994, 0.126, 0.154, 0.189, 0.227, 0.271, 0.318, 0.371, 0.43, 0.498, 0.583, 0.855, 0.952, 1.04, 1.12, 1.2, 1.28, 1.33
	};

	const short int NFlux = 20;
	double XFlux[NFlux] = {
	0.0024, 0.0048, 0.0096, 0.0168, 0.0264, 0.0408, 0.06, 0.0864, 0.122, 0.168, 0.228, 0.3, 0.389, 0.497, 0.624, 0.775, 0.953, 1.01, 1.08, 1.2
	};
	double YFlux[NFlux] = {
	0.0391, 0.0554, 0.0783, 0.104, 0.13, 0.161, 0.196, 0.235, 0.279, 0.327, 0.382, 0.438, 0.498, 0.563, 0.631, 0.704, 0.78, 0.805, 0.848, 0.923
	};

	tab_depthUsual.setTables(XUsual,YUsual,NUsual);
	tab_depthFlux.setTables(XFlux,YFlux,NFlux);
	
	return EXIT_SUCCESS;
}

int skinDepth::setTabSin(void) 
{
	const short int NUsual = 21;
	double XUsual[NUsual] = {
	0.05, 0.1, 0.2, 0.35, 0.85, 1, 1.15, 1.25, 1.35, 1.4, 1.45, 1.5, 1.55, 1.6, 1.65, 1.7, 1.75, 1.8, 1.85, 1.9, 1.95
	};
	double YUsual[NUsual] = {
	0.203, 0.287, 0.408, 0.547, 0.951, 1.09, 1.26, 1.4, 1.57, 1.66, 1.76, 1.88, 2.01, 2.16, 2.33, 2.52, 2.74, 3, 3.33, 3.75, 4.41
	};

	const short int NFlux = 12;
	double XFlux[NFlux] = {
	0.05, 0.1, 0.2, 0.35, 0.65, 1.1, 1.3, 1.5, 1.65, 1.75, 1.85, 1.95
	};
	double YFlux[NFlux] = {
	0.183, 0.259, 0.367, 0.489, 0.685, 0.971, 1.12, 1.3, 1.47, 1.6, 1.76, 1.94
	};

	tab_depthUsual.setTables(XUsual,YUsual,NUsual);
	tab_depthFlux.setTables(XFlux,YFlux,NFlux);
	
	return EXIT_SUCCESS;
}

int skinDepth::setTabSin2(void) 
{
	const short int NUsual = 28;
	double XUsual[NUsual] = {
	0.004, 0.016, 0.048, 0.12, 0.26, 0.736, 0.912, 1.044, 1.152, 1.244, 1.324, 1.4, 1.468, 1.532, 1.592, 1.644, 1.692, 1.736, 1.776, 1.808, 1.836, 1.86, 1.88, 1.9, 1.916, 1.924, 1.94, 1.96
	};
	double YUsual[NUsual] = {
	0.0449, 0.0898, 0.156, 0.247, 0.368, 0.703, 0.859, 1.01, 1.17, 1.34, 1.53, 1.74, 1.97, 2.21, 2.49, 2.76, 3.04, 3.33, 3.64, 3.92, 4.19, 4.45, 4.7, 4.98, 5.24, 5.39, 5.72, 6.26
	};

	const short int NFlux = 13;
	double XFlux[NFlux] = {
	0.004, 0.02, 0.06, 0.148, 0.32, 0.916, 1.14, 1.3, 1.46, 1.6, 1.77, 1.94, 1.96
	};
	double YFlux[NFlux] = {
	0.0414, 0.0926, 0.16, 0.253, 0.376, 0.726, 0.89, 1.05, 1.23, 1.44, 1.71, 2.02, 2.05
	};

	tab_depthUsual.setTables(XUsual,YUsual,NUsual);
	tab_depthFlux.setTables(XFlux,YFlux,NFlux);
	
	return EXIT_SUCCESS;
}

void skinDepth::setParameters(QString shape_selection,QString material_selection) 
{
	if(shape_selection=="Ramp") setTabRamp();
	if(shape_selection=="Sinus") setTabSin();
	if(shape_selection=="Sinus^2") setTabSin2();
	
	if(material_selection=="Aluminum") rho=2.709e-8;
	if(material_selection=="Brass") rho=6.29e-8;
	if(material_selection=="Copper") rho=1.712e-8;
	if(material_selection=="Gold") rho=2.255e-8;
	if(material_selection=="Silver") rho=1.617e-8;
	if(material_selection=="Stainless Steel") rho=66.3e-8;
	if(material_selection=="Tungsten") rho=5.39e-8;
	
}

double skinDepth::f(void)
{	
	double Kusual, Kflux;
	double d, t, Tm;
	int statusUsual;
	int statusFlux;
	
	Tm=parameters[1];
	t=parameters[2];
	d=parameters[3];
	
	statusUsual=tab_depthUsual.getValue((t/Tm),tabulation::InterpLinear);
	Kusual=tab_depthUsual.value;
	statusFlux=tab_depthFlux.getValue((t/Tm),tabulation::InterpLinear);
	Kflux=tab_depthFlux.value;
	delta=250*sqrt(10*rho*t)*Kusual;
	deltaFlux=250*sqrt(10*rho*t)*Kflux;
	message=QString("resistivity=%1 ohm*m<br>flux skin depth=%2 m").arg(rho,0,'g',PRECISION).arg(deltaFlux,0,'g',PRECISION);
	if((statusUsual==2) || (statusFlux==2)) message.append(" - Warning: Out of tables!");
	
	return d-delta;
}
